require 'json'

class NodeVersion < Array
  def initialize(list)
    super(list.split('.').map { |e| e.to_i })
  end

  def <(cmp)
    (self <=> cmp) < 0
  end

  def >(cmp)
    (self <=> cmp) > 0
  end

  def ==(cmp)
    (self <=> cmp).zero?
  end
end

class NodeJSListStore
  @@list = nil

  def self.add(list)
    @@list = list
  end

  def self.cached?
    @@list.nil? == false
  end

  def self.get
    @@list
  end
end

# inspired by https://github.com/visionmedia/n/blob/5630984059fb58f47def8dca2f25163456181ed3/bin/n#L363-L372
def get_version_list
  return NodeJSListStore::get if NodeJSListStore::cached?

  uri = URI('https://nodejs.org/dist/index.json')

  http_proxy = ENV['http_proxy']
  if http_proxy.to_s != ''
    if http_proxy =~ %r{/^http[s]{0,1}:\/\/.*/}
      proxy = URI.parse(http_proxy)
    else
      proxy = URI.parse('http://' + http_proxy)
    end
    request = Net::HTTP::Proxy(proxy.host, proxy.port).new(uri.host, uri.port)
  else
    request = Net::HTTP.new(uri.host, uri.port)
  end
  request.use_ssl = (uri.scheme == 'https')
  request.open_timeout = 2
  request.read_timeout = 2

  result = JSON.parse(request.get(uri.request_uri).body)
  NodeJSListStore::add(result)

  result
end

def get_latest_version
  version_data = get_version_list.map { |o| o['version'].gsub(/^v/, '') }
  version_data.sort! { |a, b| NodeVersion.new(a) <=> NodeVersion.new(b) }
  'v' + version_data.last
end

def get_lts_version
  # in this case it needs to be checked whether NOT false as the `lts` value can either be `false`
  # or the name of the LTS (e.g. Argon)
  version_data = get_version_list.select { |o| o['lts'] != false }
  version_data.first['version']
end

def get_version_from_branch(version)
  if version =~ /^[0-9]+\.x$/
    version.gsub!(/\.x$/, '')
    regex = /^v#{version}\.[0-9]+\.[0-9]+$/
  else
    regex = /^v#{version}\.([0-9]+)$/
  end

  version_data = get_version_list
                 .map { |o| o['version'] }
                 .select { |v| v =~ regex }

  version_data.first
end
